
/*! yuzu-loadable - v2.0.0-rc.14
 * https://github.com/dwightjack/yuzu/packages/loadable
 * Copyright (c) 2019 - Marco Solazzi;
 * Licensed MIT
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var yuzuUtils = require('yuzu-utils');
var yuzu = require('yuzu');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

/**
 * A function that returns a configurable async component loader.
 *
 * @param {object} opts
 * @param {Component} opts.component Component to initialize when `opts.fetchData` is resolved
 * @param {function} [opts.fetchData] A function to load remote data. Must return a promise
 * @param {function} [opts.template] Component template. A function returning a string
 * @param {Component} [opts.loader] Loader component. Shown during `opts.fetchData` execution
 * @param {string|function} [opts.renderRoot='div'] Tag used for the element holding the async component. Either a string or a function returning a DOM element.
 * @param {object} [opts.options] Component options
 * @param {props} [opts.props] Computed state attached to the component
 * @returns {LoadableComponent} Component constructor
 * @example
 *
 * import { Component } from 'yuzu';
 * import { Loadable } from 'yuzu-loadable';
 *
 * class Message extends Component {}
 * class Loader extends Component {}
 *
 * const delay = () => new Promise((resolve) => {
 *   setTimeout(() => {
 *     resolve({ message: 'Hello World' });
 *   }, 1000);
 * });
 *
 * const template = (props) => `<div class="Message">${props.message}</div>`;
 *
 * const LoadableMessage = Loadable({
 *   component: Message,
 *   loader: Loader,
 *   fetchData: delay,
 *   template
 * });
 *
 * const message = new LoadableMessage().mount('#loadable-message');
 */
function Loadable(opts) {
    var _a;
    var Child = opts.component, params = __rest(opts, ["component"]);
    var childName = Child.displayName || Child.name || '';
    /**
     * Async component loader
     *
     * @class
     * @extends Component
     * @param {object} [config]
     * @param {Component} config.component Component to initialize when `config.fetchData` is resolved
     * @param {function} [config.fetchData] A function to load remote data. Must return a promise
     * @param {function} [config.template] Component template. A function returning a string
     * @param {Component} [config.loader] Loader component. Shown during `config.fetchData` execution
     * @param {string|function} [config.renderRoot='div'] Tag used for the element holding the async component. Either a string or a function returning a DOM element.
     * @param {object} [config.options] Component options
     * @param {object|function} [config.props] Computed state attached to the component
     * @returns {LoadableComponent}
     */
    var LoadableComponent = (_a = /** @class */ (function (_super) {
            __extends(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.state = {
                    props: {},
                };
                return _this;
            }
            class_1.prototype.defaultOptions = function () {
                return Object.assign({
                    fetchData: yuzuUtils.noop,
                    component: Child,
                    template: yuzuUtils.noop,
                    loader: null,
                    options: {},
                    renderRoot: 'div',
                    props: function (v) { return v; },
                }, params);
            };
            /**
             * Initialize hook.
             *
             * Will replace the current root element contents with an empty element used as root for both the optional loader (passed as `config.loader` to the constructor)
             * the rendered component root.
             *
             * It will then set the loader (if available) and fetch the data (`config.fetchData`) before initializing the async component.
             *
             * @async
             * @memberof LoadableComponent
             * @returns {LoadableComponent}
             */
            class_1.prototype.initialize = function () {
                return __awaiter(this, void 0, void 0, function () {
                    var _a, fetchData, renderRoot, $el, async, data, root, e_1;
                    return __generator(this, function (_b) {
                        switch (_b.label) {
                            case 0:
                                _a = this.options, fetchData = _a.fetchData, renderRoot = _a.renderRoot;
                                $el = this.$el;
                                // empty the component
                                $el.textContent = '';
                                if (renderRoot) {
                                    async =
                                        typeof renderRoot === 'string'
                                            ? document.createElement(renderRoot)
                                            : renderRoot($el);
                                }
                                else {
                                    throw new TypeError('"options.renderRoot" must be either a function or a string');
                                }
                                this.$els.async = $el.appendChild(async);
                                return [4 /*yield*/, this.setLoader()];
                            case 1:
                                _b.sent();
                                _b.label = 2;
                            case 2:
                                _b.trys.push([2, 6, , 7]);
                                return [4 /*yield*/, fetchData(this)];
                            case 3:
                                data = _b.sent();
                                if (data) {
                                    this.setState({ props: data });
                                }
                                return [4 /*yield*/, this.render()];
                            case 4:
                                root = _b.sent();
                                return [4 /*yield*/, this.setComponent(root)];
                            case 5:
                                _b.sent();
                                return [2 /*return*/, this];
                            case 6:
                                e_1 = _b.sent();
                                console.error(e_1);
                                return [2 /*return*/, this];
                            case 7: return [2 /*return*/];
                        }
                    });
                });
            };
            /**
             * Initializes the rendered component as child of the loadable instance.
             *
             * The component will be attached as child into `$refs.async`.
             *
             * @memberof LoadableComponent
             * @param {Element} [root] Component root element. Will default to `$els.async` if not defined.
             * @return {Promise}
             * @example
             * class Loader extends Component {
             *   // ...
             * }
             *
             * const LoadableMessage = Loadable({
             *   component: Message,
             * });
             *
             * const loadable = new LoadableMessage();
             * loadable.setComponent()
             *
             * instanceof loadable.$refs.async === Message
             */
            class_1.prototype.setComponent = function (root) {
                var _a = this.options, component = _a.component, options = _a.options, props = _a.props;
                return this.setRef(__assign({ el: yuzuUtils.isElement(root) ? root : this.$els.async, id: 'async', component: component }, options), yuzuUtils.evaluate(props, this.state.props));
            };
            /**
             * Initializes a loader.
             *
             * If defined, will initialize the loader component set in `config.loader`,
             * else will return a resolved promise.
             *
             * The loader will be attached as child into `$refs.async`.
             *
             * @memberof LoadableComponent
             * @returns {Promise}
             * @example
             *
             * class Loader extends Component {
             *   // ...
             * }
             *
             * const LoadableMessage = Loadable({
             *   component: Message,
             *   loader: Loader
             * });
             *
             * const loadable = new LoadableMessage();
             * loadable.setLoader()
             *
             * instanceof loadable.$refs.async === Loader
             */
            class_1.prototype.setLoader = function () {
                return __awaiter(this, void 0, void 0, function () {
                    var loader;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                loader = this.options.loader;
                                if (!loader) return [3 /*break*/, 2];
                                return [4 /*yield*/, this.setRef({
                                        id: 'async',
                                        el: this.$els.async,
                                        component: loader,
                                    })];
                            case 1: return [2 /*return*/, _a.sent()];
                            case 2: return [2 /*return*/];
                        }
                    });
                });
            };
            /**
             * Renders an optional template.
             *
             * Executes the template function set in `config.template` passing an object containing a `props`
             * key equal to the `state.props` key.
             *
             * @memberof LoadableComponent
             * @returns {string|null} returns null if the template function returns a falsy value
             * @example
             * const template = (props) => `<div>${props.message}</div>`;
             *
             * const LoadableMessage = Loadable({
             *   component: Message,
             *   template
             * });
             *
             * const loadable = new LoadableMessage();
             * loadable.setState({ props: { message: 'Hello World' }});
             * const html = loadable.render();
             * html === '<div>Hello World</div>';
             */
            class_1.prototype.render = function () {
                var template = this.options.template;
                var props = this.state.props;
                var wrapper = document.createElement('div');
                var html = template && template({ props: props, options: this.options });
                if (html) {
                    wrapper.innerHTML = html;
                    if (wrapper.childElementCount > 1) {
                        this.$warn('Multi-root templates are not supported. Just the first root element will be rendered.');
                    }
                    return wrapper.firstElementChild;
                }
                return null;
            };
            return class_1;
        }(yuzu.Component)),
        _a.root = "[data-loadable=\"" + childName + "\"], [data-loadable][data-component=\"" + childName + "\"]",
        _a);
    Object.defineProperty(LoadableComponent, 'displayName', {
        value: "Loadable" + (childName || 'Component'),
    });
    return LoadableComponent;
}

exports.Loadable = Loadable;
//# sourceMappingURL=index.js.map
